from typing import List, Optional, Any
from datetime import datetime
from fastapi_camelcase import CamelModel
from telus_bulk.models.reports import ReportAddress


class ReportQualification(CamelModel):
    csq_id: Optional[str] = None
    product_type: str
    created_at: Optional[datetime | str | Any] = None
    address: List[ReportAddress]



from typing import TypeVar, Generic, List
from telus_bulk.pagination.page_meta_dto import PageMetaDto
from fastapi_camelcase import CamelModel

T = TypeVar("T")


class PageDto(CamelModel):
    def __init__(self, data: List[T], meta: PageMetaDto):
        super().__init__(data=data, meta=meta)

    data: List[T]
    meta: PageMetaDto



from fastapi_camelcase import CamelModel
import math

from pydantic import Field


class PageMetaDto(CamelModel):
    def __init__(self, page: int, limit: int, item_count: int):
        page_count = math.ceil(item_count / limit)
        super().__init__(
            page=page,
            page_size=limit,
            item_count=item_count,
            page_count=page_count,
            has_previous_page=page > 1,
            has_next_page=page < page_count,
        )

    page: int = Field(..., ge=1)
    page_size: int
    item_count: int
    page_count: int
    has_previous_page: bool
    has_next_page: bool



from fastapi_camelcase import CamelModel
from typing import List
from loguru import logger
from google.cloud.firestore_v1.document import DocumentReference


class FirestoreReference(CamelModel):
    reference_id: str
    path: str


def manage_document_references(
    references: List[DocumentReference],
) -> List[FirestoreReference]:
    firestore_references: List[FirestoreReference] = []
    for ref in references:
        try:
            firestore_references.append(
                FirestoreReference.parse_obj(
                    {
                        "reference_id": ref.id,
                        "path": ref.path,
                    }
                )
            )
        except Exception:
            logger.warning("Invalid reference")
    return firestore_references



# coding: utf-8

from __future__ import annotations

import re  # noqa: F401
from datetime import date, datetime  # noqa: F401
from typing import Any, Dict, List, Optional  # noqa: F401

from fastapi_camelcase import CamelModel
from pydantic import AnyUrl, Field  # noqa: F401


class RelatedParty(CamelModel):
    """NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).

    Do not edit the class manually.

    RelatedParty - a model defined in OpenAPI

        id: The id of this RelatedParty.
        href: The href of this RelatedParty [Optional].
        name: The name of this RelatedParty [Optional].
        role: The role of this RelatedParty [Optional].
        base_type: The base_type of this RelatedParty [Optional].
        schema_location: The schema_location of this RelatedParty [Optional].
        type: The type of this RelatedParty [Optional].
        referred_type: The referred_type of this RelatedParty.
    """

    id: Optional[str] = Field(default="1")
    href: Optional[AnyUrl] = None
    name: Optional[str] = None
    role: Optional[str] = None
    base_type: Optional[str] = None
    schema_location: Optional[AnyUrl] = None
    type: Optional[str] = Field(default=None, alias="@type")
    referred_type: Optional[str] = Field(default=None, alias="@referredType")


RelatedParty.update_forward_refs()



# coding: utf-8

from __future__ import annotations

import re  # noqa: F401
from datetime import date, datetime  # noqa: F401
from typing import Any, Dict, List, Optional  # noqa: F401

from fastapi_camelcase import CamelModel  # noqa: F401


class EventSubscription(CamelModel):
    """NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).

    Do not edit the class manually.

    EventSubscription - a model defined in OpenAPI

        id: The id of this EventSubscription.
        callback: The callback of this EventSubscription.
        query: The query of this EventSubscription [Optional].
    """

    id: str
    callback: str
    query: Optional[str] = None


EventSubscription.update_forward_refs()



from typing import Any, Optional

from fastapi_camelcase import CamelModel
from pydantic import Field


class Coordinate(CamelModel):
    srs_code: str = Field(..., alias="srsCode")
    latitude: float
    longitude: float
    location_accuracy_score: Optional[str] = Field(
        default=None, alias="locationAccuracyScore"
    )
    location_provider: Optional[Any] = Field(default=None, alias="locationProvider")
    location_update_ts: Optional[str] = Field(default=None, alias="locationUpdateTs")



from __future__ import annotations

from typing import Optional, Literal

from fastapi_camelcase import CamelModel
from pydantic import Field
from telus_bulk.models.ams.coordinate import Coordinate
from telus_bulk.models.clli import Clli


class Place(CamelModel):
    id: str
    role: Optional[str] = None
    _type: str = Field(alias="@type", default="GeographicAddress")


class PlaceAMS(Place):
    city: Optional[str] = None
    country: Literal["USA", "CAN"] = "CAN"
    postcode: Optional[str] = None
    state_or_province: Optional[str] = None
    street_dir: Optional[str] = None
    street_name: Optional[str] = None
    street_nr: Optional[str] = None
    street_type: Optional[str] = None
    street_type_suffix: Optional[str] = None
    street_type_prefix: Optional[str] = None
    street_number_prefix: Optional[str] = None
    street_number_suffix: Optional[str] = None
    dir_prefix: Optional[str] = None
    dir_suffix: Optional[str] = None
    floor: Optional[str] = None
    unit: Optional[str] = None
    address_type: Optional[str] = None
    coordinate: Optional[Coordinate] = None
    clli: Optional[Clli] = Field(default=None)



from typing import Any, List, Dict, Optional, Literal

from fastapi_camelcase import CamelModel
from pydantic import Field

from telus_bulk.models.ams.coordinate import Coordinate
from telus_bulk.models.ams.e911_address import E911Address
from telus_bulk.models.ams.fms_address import FmsAddress
from telus_bulk.models.ams.reference_id import ReferenceId
from telus_bulk.models.ams.reference_ids import ReferenceIds
from telus_bulk.models.ams.zone_info_item import ZoneInfoItem
from telus_bulk.models.worker_job.place import PlaceAMS


class AddressDetailModel(CamelModel):
    address_id: str
    address_type: Optional[str]
    reference_id: Optional[ReferenceId]
    reference_ids: Optional[ReferenceIds]
    unit: Any
    floor: Any
    street_number_prefix: Any
    street_number: Optional[str]
    street_number_suffix: Any
    dir_prefix: Any
    street_type_prefix: Any
    street_name: Optional[str]
    street_type_suffix: Any
    dir_suffix: Any
    city: Optional[str]
    province: str
    postal_code: Optional[str] = Field(default="N/A")
    country: Optional[Literal["USA", "CAN"]] = Field(default="CAN")
    coid: Optional[str]
    serviceable: Optional[str]
    serviceable_code: Optional[str]
    building_type: Optional[str]
    premise_count_flag: Optional[Any]
    premise_count_date: Optional[Any]
    postal_code_updt_ts: Optional[Any]
    res_line_count: Optional[str]
    bus_line_count: Optional[str]
    mdu_sfu_count: Optional[str]
    project_sub_type: Optional[Any]
    first_nation_ind: Optional[str]
    exception_code: Optional[Any]
    nap_addr_assoc: Optional[Any]
    coordinate: Optional[Coordinate]
    zone_info: Optional[List[ZoneInfoItem]]
    fms_address: Optional[FmsAddress]
    e911_address: Optional[E911Address]
    tag: Optional[List]
    location_comments: Optional[Dict[str, Any]]

    def to_csq_item(self) -> PlaceAMS:
        return PlaceAMS(
            id=self.address_id,
            city=self.city,
            country=self.country,
            postcode=self.postal_code,
            state_or_province=self.province,
            street_dir=self.dir_prefix,
            street_name=self.street_name,
            street_nr=self.street_number,
            street_type=self.street_type_prefix,
            street_type_suffix=self.street_type_suffix,
            street_type_prefix=self.street_type_prefix,
            street_number_prefix=self.street_number_prefix,
            street_number_suffix=self.street_number_suffix,
            dir_suffix=self.dir_suffix,
            dir_prefix=self.dir_prefix,
            floor=self.floor,
            unit=self.unit,
            address_type=self.address_type,
            coordinate=self.coordinate,
        )



from typing import Optional

from fastapi_camelcase import CamelModel
from pydantic import Field


class Clli(CamelModel):
    city: Optional[str] = Field(default=None)
    province: Optional[str] = Field(default=None)
    clli: Optional[str] = Field(default=None)
    distance: Optional[str] = Field(default=None)



from datetime import datetime
from enum import Enum
from typing import Any, Optional

from fastapi_camelcase import CamelModel


class OverallProcessorType(Enum):
    """
    Statis for locations objects in over_all_process collection
    """

    QUICK = "quick"
    FULL = "full"

    def __str__(self) -> str:
        return str(self.value)


class OverallProcessStatus(Enum):
    """
    Statis for locations objects in over_all_process collection
    """

    PENDING = "pending"
    SENT = "sent"
    DONE = "done"
    PROCESSING = "processing"

    def __str__(self) -> str:
        return str(self.value)


class OverallProcess(CamelModel):
    """
    docstring
    """

    location: Optional[str] = None
    status: Optional[str] = OverallProcessStatus.PENDING.value
    csq_id: Optional[str] = None
    csq_ref: Optional[str] = None
    id: Optional[str] = None
    created_at = datetime.now()


OverallProcess.update_forward_refs()


class OverallData(CamelModel):
    """
    docstring
    """

    location_id: Optional[str] = None
    location_obj: Optional[dict] = None
    partner: Optional[str] = None
    to_whom: Optional[str] = None
    offers: Optional[list[dict]] = []
    csq_id: Optional[str] = None
    csq_ref: Optional[str] = None
    id: Optional[str] = None
    status: Optional[str] = None
    created_at: Optional[datetime | Any] = None
    updated_at: Optional[str | Any] = None
    created_at_fancy: Optional[str | Any] = None
    updated_at_fancy: Optional[str | Any] = None
    processor_type: Optional[str] = None


OverallData.update_forward_refs()



from typing import List, Optional
from fastapi_camelcase import CamelModel
from telus_bulk.models.worker_job.address_processing_job import AddressProcessingJob


class RelatedParty(CamelModel):
    partner_name: str
    contact_method: str
    contact_email: Optional[str] = None


class CityCoverageProcessedJob(AddressProcessingJob):
    related_party: List[RelatedParty] = []



from typing import Optional
from fastapi_camelcase import CamelModel


class ReportAddress(CamelModel):
    location_id: str
    address: str
    best_match_address: str
    qual_status: Optional[str] = None
    carrier: Optional[str] = None
    best_offer: Optional[str] = None
    speed: Optional[str] = None
    technology: Optional[str] = None
    region_clli: Optional[str] = None
    nearest_distance: Optional[str] = None
    location_type: Optional[str] = None
    comments: Optional[str] = None


class ReportAddressUpdateDto(CamelModel):
    address: Optional[str]
    best_match_address: Optional[str]
    qual_status: Optional[str] = None
    carrier: Optional[str] = None
    best_offer: Optional[str] = None
    speed: Optional[str] = None
    technology: Optional[str] = None
    region_clli: Optional[str] = None
    nearest_distance: Optional[str] = None
    location_type: Optional[str] = None
    comments: Optional[str] = None



# coding: utf-8

from __future__ import annotations

import re  # noqa: F401
from datetime import date, datetime  # noqa: F401
from typing import Any, Dict, List, Optional  # noqa: F401

from fastapi_camelcase import CamelModel
from pydantic import AnyUrl, Field  # noqa: F401

from telus_bulk.models.tmf_645.characteristic import Characteristic
from telus_bulk.models.tmf_645.feature import Feature
from telus_bulk.models.tmf_645.note import Note
from telus_bulk.models.tmf_645.related_entity_ref_or_value import (
    RelatedEntityRefOrValue,
)
from telus_bulk.models.tmf_645.related_party import RelatedParty
from telus_bulk.models.tmf_645.related_place_ref_or_value import RelatedPlaceRefOrValue
from telus_bulk.models.tmf_645.related_service_order_item import RelatedServiceOrderItem
from telus_bulk.models.tmf_645.resource_ref import ResourceRef

# from telus_bulk.models.tmf_645.service_relationship import ServiceRelationship
from telus_bulk.models.tmf_645.service_specification_ref import ServiceSpecificationRef
from telus_bulk.models.tmf_645.service_state_type import ServiceStateType
from telus_bulk.models.worker_job.place import PlaceAMS
from telus_bulk.models.worker_job.service_specification import ServiceSpecification


class ServiceRefOrValue(CamelModel):
    """NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).

    Do not edit the class manually.

    ServiceRefOrValue - a model defined in OpenAPI

        id: The id of this ServiceRefOrValue [Optional].
        href: The href of this ServiceRefOrValue [Optional].
        category: The category of this ServiceRefOrValue [Optional].
        description: The description of this ServiceRefOrValue [Optional].
        end_date: The end_date of this ServiceRefOrValue [Optional].
        has_started: The has_started of this ServiceRefOrValue [Optional].
        is_bundle: The is_bundle of this ServiceRefOrValue [Optional].
        is_service_enabled: The is_service_enabled of this ServiceRefOrValue [Optional].
        is_stateful: The is_stateful of this ServiceRefOrValue [Optional].
        name: The name of this ServiceRefOrValue [Optional].
        service_date: The service_date of this ServiceRefOrValue [Optional].
        service_type: The service_type of this ServiceRefOrValue [Optional].
        start_date: The start_date of this ServiceRefOrValue [Optional].
        start_mode: The start_mode of this ServiceRefOrValue [Optional].
        feature: The feature of this ServiceRefOrValue [Optional].
        note: The note of this ServiceRefOrValue [Optional].
        place: The place of this ServiceRefOrValue [Optional].
        related_entity: The related_entity of this ServiceRefOrValue [Optional].
        related_party: The related_party of this ServiceRefOrValue [Optional].
        service_characteristic: The service_characteristic of this ServiceRefOrValue [Optional].
        service_order_item: The service_order_item of this ServiceRefOrValue [Optional].
        service_relationship: The service_relationship of this ServiceRefOrValue [Optional].
        service_specification: The service_specification of this ServiceRefOrValue [Optional].
        state: The state of this ServiceRefOrValue [Optional].
        supporting_resource: The supporting_resource of this ServiceRefOrValue [Optional].
        supporting_service: The supporting_service of this ServiceRefOrValue [Optional].
        base_type: The base_type of this ServiceRefOrValue [Optional].
        schema_location: The schema_location of this ServiceRefOrValue [Optional].
        type: The type of this ServiceRefOrValue [Optional].
        referred_type: The referred_type of this ServiceRefOrValue [Optional].
    """

    id: Optional[str] = None
    href: Optional[str] = None
    category: Optional[str] = None
    description: Optional[str] = None
    end_date: Optional[datetime] = None
    has_started: Optional[bool] = None
    is_bundle: Optional[bool] = None
    is_service_enabled: Optional[bool] = None
    is_stateful: Optional[bool] = None
    name: Optional[str] = None
    service_date: Optional[str] = None
    service_type: Optional[str] = None
    start_date: Optional[datetime] = None
    start_mode: Optional[str] = None
    feature: Optional[List[Feature]] = None
    note: Optional[List[Note]] = None
    place: Optional[List[PlaceAMS] | List[RelatedPlaceRefOrValue]] = None
    related_entity: Optional[List[RelatedEntityRefOrValue]] = None
    related_party: Optional[List[RelatedParty] | RelatedParty] = None
    service_characteristic: Optional[
        List[Characteristic] | List[Any] | Characteristic
    ] = None
    service_order_item: Optional[List[RelatedServiceOrderItem]] = None
    service_relationship: Optional[List[ServiceRelationship]] = None
    service_specification: Optional[
        List[ServiceSpecificationRef]
        | List[ServiceSpecification]
        | ServiceSpecification
    ] = None
    state: Optional[ServiceStateType] = None
    supporting_resource: Optional[List[ResourceRef]] = None
    supporting_service: Optional[List[ServiceRefOrValue]] = None
    base_type: Optional[str] = None
    schema_location: Optional[AnyUrl] = None
    type: Optional[str] = Field(default=None, alias="@type")
    referred_type: Optional[str] = Field(default=None, alias="@referredType")


class ServiceRelationship(CamelModel):
    """NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).

    Do not edit the class manually.

    ServiceRelationship - a model defined in OpenAPI

        relationship_type: The relationship_type of this ServiceRelationship.
        service: The service of this ServiceRelationship [Optional].
        service_relationship_characteristic: The service_relationship_characteristic of this ServiceRelationship [Optional].
        base_type: The base_type of this ServiceRelationship [Optional].
        schema_location: The schema_location of this ServiceRelationship [Optional].
        type: The type of this ServiceRelationship [Optional].
    """

    relationship_type: str
    service: Optional[ServiceRefOrValue] = None
    service_relationship_characteristic: Optional[List[Characteristic]] = None
    base_type: Optional[str] = None
    schema_location: Optional[AnyUrl] = None
    type: Optional[str] = Field(default=None, alias="@type")


ServiceRefOrValue.update_forward_refs()



from typing import Optional

from pydantic import BaseModel  # noqa: F401


class ExampleModel(BaseModel):
    id: Optional[str] = None
    msg: str = "Hello World!"


ExampleModel.update_forward_refs()



from email.policy import default
from typing import Optional

from fastapi_camelcase import CamelModel
from pydantic import Field

from telus_bulk.models.ams.zone_attribute import ZoneAttributes


class ZoneInfoItem(CamelModel):
    zone_type: Optional[str] = Field(default=None, alias="zoneType")
    zone_name: Optional[str] = Field(default=None, alias="zoneName")
    zone_attributes: Optional[ZoneAttributes] = Field(None, alias="zoneAttributes")



# coding: utf-8

from __future__ import annotations

import re  # noqa: F401
from datetime import date, datetime  # noqa: F401
from typing import Any, Dict, List, Optional  # noqa: F401

from fastapi_camelcase import CamelModel
from pydantic import AnyUrl, Field


class ServiceEligibilityUnavailabilityReason(CamelModel):
    """NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).

    Do not edit the class manually.

    ServiceEligibilityUnavailabilityReason - a model defined in OpenAPI

        code: The code of this ServiceEligibilityUnavailabilityReason [Optional].
        label: The label of this ServiceEligibilityUnavailabilityReason [Optional].
        base_type: The base_type of this ServiceEligibilityUnavailabilityReason [Optional].
        schema_location: The schema_location of this ServiceEligibilityUnavailabilityReason [Optional].
        type: The type of this ServiceEligibilityUnavailabilityReason [Optional].
    """

    code: Optional[str] = None
    label: Optional[str] = None
    base_type: Optional[str] = None
    schema_location: Optional[AnyUrl] = None
    type: Optional[str] = Field(default=None, alias="@type")
    construction_charge: Optional[float] = None
    comments: Optional[str] = None


ServiceEligibilityUnavailabilityReason.update_forward_refs()



from enum import Enum
from typing import List


class PartnersEnum(Enum):
    BELL_PARTNER = "Bell"
    ROGERS_PARTNER = "Rogers"
    VIDEOTRON_PARTNER = "Videotron"
    SHAW_PARTNER = "Shaw"

    def __str__(self) -> str:
        return str(self.value)


available_partners_list: List[str] = [partner.value for partner in PartnersEnum]
